@@ -341,16 +341,19 @@ static int acf_curlopt_write(struct ast_channel *chan, const char *cmd, char *na
 	enum optiontype ot;
 
 	if (chan) {
+		ast_channel_lock(chan);
 		if (!(store = ast_channel_datastore_find(chan, &curl_info, NULL))) {
 			/* Create a new datastore */
 			if (!(store = ast_datastore_alloc(&curl_info, NULL))) {
 				ast_log(LOG_ERROR, "Unable to allocate new datastore.  Cannot set any CURL options\n");
+				ast_channel_unlock(chan);
 				return -1;
 			}
 
 			if (!(list = ast_calloc(1, sizeof(*list)))) {
 				ast_log(LOG_ERROR, "Unable to allocate list head.  Cannot set any CURL options\n");
 				ast_datastore_free(store);
+				ast_channel_unlock(chan);
 				return -1;
 			}
 
@@ -360,6 +363,7 @@ static int acf_curlopt_write(struct ast_channel *chan, const char *cmd, char *na
 		} else {
 			list = store->data;
 		}
+		ast_channel_unlock(chan);
 	} else {
 		/* Populate the global structure */
 		list = &global_curl_info;
@@ -472,9 +476,17 @@ static int acf_curlopt_helper(struct ast_channel *chan, const char *cmd, char *d
 		return -1;
 	}
 
-	if (chan && (store = ast_channel_datastore_find(chan, &curl_info, NULL))) {
-		list[0] = store->data;
-		list[1] = &global_curl_info;
+	if (chan) {
+		/* If we have a channel, we want to read the options set there before
+		   falling back to the global settings */
+		ast_channel_lock(chan);
+		store = ast_channel_datastore_find(chan, &curl_info, NULL);
+		ast_channel_unlock(chan);
+
+		if (store) {
+			list[0] = store->data;
+			list[1] = &global_curl_info;
+		}
 	}
 
 	for (i = 0; i < 2; i++) {
@@ -600,8 +612,6 @@ static size_t WriteMemoryCallback(void *ptr, size_t size, size_t nmemb, void *da
 	return realsize;
 }
 
-static const char * const global_useragent = "asterisk-libcurl-agent/1.0";
-
 static int curl_instance_init(void *data)
 {
 	CURL **curl = data;
@@ -612,7 +622,7 @@ static int curl_instance_init(void *data)
 	curl_easy_setopt(*curl, CURLOPT_NOSIGNAL, 1);
 	curl_easy_setopt(*curl, CURLOPT_TIMEOUT, 180);
 	curl_easy_setopt(*curl, CURLOPT_WRITEFUNCTION, WriteMemoryCallback);
-	curl_easy_setopt(*curl, CURLOPT_USERAGENT, global_useragent);
+	curl_easy_setopt(*curl, CURLOPT_USERAGENT, AST_CURL_USER_AGENT);
 
 	return 0;
 }
@@ -898,31 +908,6 @@ static struct ast_custom_function acf_curl = {
 
 static struct ast_custom_function acf_curlopt = {
 	.name = "CURLOPT",
-	.synopsis = "Set options for use with the CURL() function",
-	.syntax = "CURLOPT(<option>)",
-	.desc =
-"  cookie         - Send cookie with request [none]\n"
-"  conntimeout    - Number of seconds to wait for connection\n"
-"  dnstimeout     - Number of seconds to wait for DNS response\n"
-"  followlocation - Follow HTTP 3xx redirects (boolean)\n"
-"  ftptext        - For FTP, force a text transfer (boolean)\n"
-"  ftptimeout     - For FTP, the server response timeout\n"
-"  header         - Retrieve header information (boolean)\n"
-"  httpheader     - Add new custom http header (string)\n"
-"  httptimeout    - Number of seconds to wait for HTTP response\n"
-"  maxredirs      - Maximum number of redirects to follow\n"
-"  proxy          - Hostname or IP to use as a proxy\n"
-"  proxytype      - http, socks4, or socks5\n"
-"  proxyport      - port number of the proxy\n"
-"  proxyuserpwd   - A <user>:<pass> to use for authentication\n"
-"  referer        - Referer URL to use for the request\n"
-"  useragent      - UserAgent string to use\n"
-"  userpwd        - A <user>:<pass> to use for authentication\n"
-"  ssl_verifypeer - Whether to verify the peer certificate (boolean)\n"
-"  hashcompat     - Result data will be compatible for use with HASH()\n"
-"                 - if value is \"legacy\", will translate '+' to ' '\n"
-"  failurecodes   - A comma separated list of HTTP response codes to be treated as errors\n"
-"",
 	.read = acf_curlopt_read,
 	.read2 = acf_curlopt_read2,
 	.write = acf_curlopt_write,
