@@ -77,6 +77,13 @@
 		</synopsis>
 		<syntax>
 			<parameter name="filenames" required="true" argsep="&amp;">
+				<para>Ampersand separated list of filenames. If the filename
+				is a relative filename (it does not begin with a slash), it
+				will be searched for in the Asterisk sounds directory. If the
+				filename is able to be parsed as a URL, Asterisk will
+				download the file and then begin playback on it. To include a
+				literal <literal>&amp;</literal> in the URL you can enclose
+				the URL in single quotes.</para>
 				<argument name="filename1" required="true" />
 				<argument name="filename2" multiple="true" />
 			</parameter>
@@ -1251,7 +1258,7 @@ static int pbx_builtin_background(struct ast_channel *chan, const char *data)
 
 		ast_stopstream(chan);		/* Stop anything playing */
 		/* Stream the list of files */
-		while (!res && (front = strsep(&back, "&")) ) {
+		while (!res && (front = ast_strsep(&back, '&', AST_STRSEP_STRIP | AST_STRSEP_TRIM))) {
 			if ( (res = ast_streamfile(chan, front, args.lang)) ) {
 				ast_log(LOG_WARNING, "ast_streamfile failed on %s for %s\n", ast_channel_name(chan), (char*)data);
 				res = 0;
